//*****************************************************************************
//-----------------------------------------------------------------------------
// Fast simulation and reconstruction package for TOP counter (F77 core)
// M. Staric, July-2009
//-----------------------------------------------------------------------------
//
// top_simu.cc
// example of using the code: simulation and reconstruction of uniformly 
//                            generated muons, pions and kaons into solid angle
//                            and momenta uniformly from 0 to 5 GeV/c
//-----------------------------------------------------------------------------
//*****************************************************************************

#include <iostream>
#include <iomanip>
#include <math.h>
#include "TOPsimu.h"
#include "TOPreco.h"
#include "RandomParticle.h"
#include "TOPhbook.h"

void TOPconfigure(int geo, int PMTchoice, int QEchoice, double Redge);
double Masses[3]={0.10566, 0.13957, 0.49368}; 
int HypID[3]={2,3,4};
double LogL[3], ExpPhot[3]; int Nphot;
int Nhyp=3;

using namespace std;

int main()
{
  int geo=2;
  int PMTchoice=1;
  int QEchoice=3;
  double Redge=0.3e-1;
  double sigT0=25e-3;
  double Bkg=10;
  char HbookFile[80]="../hbo/top_reco.hbook";
  int ngen=1000;

  TOPconfigure(geo, PMTchoice, QEchoice, Redge);
  TOPsimu simu(sigT0, Bkg);
  TOPreco reco(Nhyp, Masses);
  reco.SetHypID(Nhyp, HypID);
  RandomParticle p;
  p.setFrac(0, 0, 1, 1, 0);
  //  p.setPmin(3.);
  //  p.setPmax(3.);
  p.setCosThmin(-0.6);
  p.setCosThmax(0.9);

  HbookOpen(HbookFile);
  hbook1(2000,"photons: t (ns)",500, 0, 50);
  hbook1(2001,"photons: t-min(t0) (ns)", 200, -10, 10);
  hbook1(2002,"photons: t-t0 (ps)", 100, -500, 500);
  hbook1(2003,"photons: (t-t0)/sigma", 100, -10, 10);
  hbprof(2004,"photons: t-t0 vs. PhiCer", 90, -180, 180, -500, 500);
  hbprof(2005,"photons: (t-t0)/sigma vs. PhiCer", 90, -180, 180, -10, 10, "S");
  hbook2(2006,"measured vs. expected photons for pions",100,0,100,100,0,100);
  hbook2(2007,"measured vs. expected photons for kaons",100,0,100,100,0,100);
  hbook1(2008,"photons: bkg wt",101, 0, 1.01);
  hbook1(2009,"photons: t (ns) bkg wt",500, 0, 50);

  for(int iev=1; iev<=ngen; iev++){
    if(iev%1000==0) cout << "Event="<<iev<<endl;
    simu.Clear();
    simu.AddParticle(p);
    simu.Simulate();
      
    reco.Clear();
    reco.AddData(simu);
    TOPtrack trk(p);
    trk.toTop();
    trk.smear(1.0e-1, 1.4e-1, 1.5e-3, 1.5e-3);
    reco.Reconstruct(trk);

    if(reco.Flag() != 0) {
      reco.GetLogL(Nhyp, LogL, ExpPhot, Nphot);
      NtupleInit(1,"TOP reco");
      NtupleFill("iev", iev );
      NtupleFill("lund", p.Lund() );
      NtupleFill("type", p.Hyp() );
      NtupleFill("charge", p.Q() );
      NtupleFill("p", p.p() );
      NtupleFill("cth", cos(p.theta()) );
      NtupleFill("phi", p.phi() );
      NtupleFill("nph0", simu.NphExit(p.Label()));
      NtupleFill("nph1", simu.NphDigi(p.Label()));
      NtupleFill("fl", reco.Flag());
      NtupleFill("nphot", Nphot);
      NtupleFill("lmu", LogL[0]);
      NtupleFill("lpi", LogL[1]);
      NtupleFill("lk",  LogL[2]);
      NtupleFill("nphmu", ExpPhot[0]);
      NtupleFill("nphpi", ExpPhot[1]);
      NtupleFill("nphk",  ExpPhot[2]);
      double R[3], Dir[3], Tlen, TOF, pmom; int QbarID;
      reco.GetHit(Local, R, Dir, Tlen, TOF, pmom, QbarID);
      NtupleFill("qbar",QbarID);
      NtupleFill("x",R[0]);
      NtupleFill("z",R[2]);
      NtupleFill("tlen",Tlen);
      NtupleFill("tof",TOF);
      reco.GetHit(Global, R, Dir, Tlen, TOF, pmom, QbarID);
      NtupleFill("xg",R[0]);
      NtupleFill("yg",R[1]);
      NtupleFill("zg",R[2]);
      NtupleFlush();
    }

    if(reco.Flag() == 1) {
      if(p.Hyp()==3) hfill(2006,ExpPhot[1],Nphot);
      if(p.Hyp()==4) hfill(2007,ExpPhot[2],Nphot);
    }

    double t, t0, sigma, phiCer, wt;
    for(int k=0; k<reco.PullSize(); k++){
      reco.GetPull(k, t, t0, sigma, phiCer, wt);
      if(sigma>0){
	double dt=(t-t0)*1000; double pull=(t-t0)/sigma;
	hfill(2002,dt,0,wt);
	hfill(2003,pull,0,wt);
	double phi=phiCer/3.141592*180.;
	hfill(2004,phi,dt,wt);
	hfill(2005,phi,pull,wt);
      }
      else{
	hfill(2000,t);
	hfill(2001,t-t0);
	hfill(2008,wt);
	hfill(2009,t,0,wt);
      }	
    }

    p.genNext();
  }

  HbookClose();
  return 0;
}


