//*****************************************************************************
//-----------------------------------------------------------------------------
// Fast simulation and reconstruction package for TOP counter (F77 core)
// M. Staric, March-2009
//-----------------------------------------------------------------------------
//
// TOPconfig.h
// C++ interface to F77 functions: TOP counter configuration
// units: cm, ns, T, nm
//-----------------------------------------------------------------------------
//*****************************************************************************

#ifndef _TOPconfig_h
#define _TOPconfig_h

#include "TOPf77fun.h"
#include <cstring>


enum{None=0, PMT, PlaneM, CylindricM, SphericM}; // exit window types
enum{Left=0, Right=1}; // argument LR
enum{NoRefl=0, Box, Prism};  // expansion volume shapes


// define TOP counter volume (must be called first)
// R1, R2: inner/outer radius [cm]; Z1, Z2: min/max z [cm]

inline void TOPvolume(double R1, double R2, double Z1, double Z2)
{
  float r1 = R1; float r2 = R2; float z1 = Z1; float z2 = Z2;
  set_topvol_(&r1, &r2, &z1, &z2);
}

// set magnetic field [T]
// use negative value to reverse the polarity
// B=0 is also possible now

inline void setBfield( double B)
{
  float b=B;
  set_bfield_(&b);
}

// set Q-bar edge roughness (radius)

inline void setEdgeRoughness(double R)
{
  float r=R;
  set_qbar_redg_(&r);
}

// define PMT dimensions (must be called prior to setQbar)
// A, B: dimensions in x and y [cm]
// Asens, Bsens: sensitive area [cm]
// Nx, Ny: number of channels in x and y
// TTS: time resolution (sigma) [ns] for (one) gaussian PDF 

inline void setPMT(double A, double B, double Asens, double Bsens, 
		   int Nx, int Ny, double TTS=50.e-3)
{
  float a=A; float b=B; float aa=Asens; float bb=Bsens; float tts=TTS;
  set_pmt_(&a, &b, &aa, &bb, &Nx, &Ny, &tts);
}

// define TTS with multi-gaussian PDF
// ng: num. of Gaussains; Frac: fractions; Mean: means [ns]; Sigma: sigmas [ns]

inline void setTTS(int ng, double Frac[], double Mean[], double Sigma[])
{
  float frac[ng], t0[ng], sig[ng];
  for(int i=0; i<ng; i++){
    frac[i]=Frac[i]; 
    t0[i]=Mean[i]; 
    sig[i]=Sigma[i];
  }	
  set_tts_(&ng, frac, t0, sig);
}

// read quantum efficency from file
// CE: electron collection efficiency

inline void setQE(const char *file, double CE)
{
  int len=strlen(file); float ce=CE;
  read_qeffi_(file, &ce, len);
}

// set quantum efficiency
// CE: electron collection efficiency

inline void setQE(double Wavelength[], double QE[], int Size, double CE)
{
  float lam[Size], qef[Size];
  for(int i=0; i<Size; i++){
    lam[i]=Wavelength[i];
    qef[i]=QE[i];
  }
  float ce=CE;
  set_qeffi_(lam, qef, &Size, &ce);
}
     
// set quantum efficiency: GaAsP (Inami-san data)
// CE: electron collection efficiency

inline void setQE_GaAsP(double CE)
{
  float ce=CE;
  float Lambda[57] = {
    200,210,220,230,240,250,260,270,280,290,
    300,310,320,330,340,350,360,370,380,390,
    400,410,420,430,440,450,460,470,480,490,
    500,510,520,530,540,550,560,570,580,590,
    600,610,620,630,640,650,660,670,680,690,
    700,710,720,730,740,750,760};
  float Qe[57] = {
    0.14,0.21,0.22,0.22,0.19,0.16,0.26,0.81,2.04,3.48,
    4.75,5.94,7.3,8.83,10.5,12.3,13.8,15.0,16.4,17.6,
    18.8,20.8,22.6,24.2,25.7,26.9,27.9,28.7,29.5,30.0,
    30.4,30.8,30.9,30.9,30.9,30.9,31.0,30.8,30.2,29.4,
    28.1,26.7,25.7,25.2,24.7,24.0,22.8,20.9,18.2,14.5,
    9.06,3.93,1.15,0.27,0.07,0.03,0.02};
  int Size=57;
  float qef[Size];
  for(int i=0; i<Size; i++) {qef[i] = 1.33*Qe[i]/100;}
  set_qeffi_(Lambda, qef, &Size, &ce);
}
     
// set quantum efficiency: MultiAlkali (Inami-san data)
// CE: electron collection efficiency

inline void setQE_MultiAlkali(double CE)
{
  float ce=CE;
  float Qe[70] = {
    0.0, 0.0, 0.0, 0.0, 0.0,
    0.0, 11.0, 11.0, 18.0, 18.0,
    21.0, 21.0, 23.0, 23.0, 23.0,
    23.0, 24.0, 24.0, 25.0, 25.0,
    23.0, 23.0, 21.0, 21.0, 20.0,
    20.0, 17.0, 17.0, 14.0, 14.0,
    12.0, 12.0, 11.0, 11.0, 9.5,
    9.5, 8.1, 8.1, 7.0, 7.0,
    6.0, 6.0, 5.0, 5.0, 4.2,
    4.2, 3.6, 3.6, 3.0, 3.0,
    2.3, 2.3, 1.9, 1.9, 1.3,
    1.3, 0.93, 0.93, 0.6, 0.6,
    0.34, 0.34, 0.17, 0.17, 0.06,
    0.06, 0.02, 0.02, 0.004, 0.004};
  int Size=70;
  float Lambda[Size];
  float qef[Size];
  for(int i=0; i<Size; i++) {
    Lambda[i]=200+i*10;
    qef[i] = Qe[i]/100;
  }	
  set_qeffi_(Lambda, qef, &Size, &ce);
}

// define Qbar geometry; returns QbarID
// A, B: width and thickness [cm]
// Z1, Z2: exit window positions in Belle frame (Z1<Z2) [cm]     
// R: radius [cm] 
// Dx: shift along the Q-bar frame x-axis
// Phi: angle in respect to Belle y-axis [radians]
// Lside, Rside: types of the left/right exit windows, see enum{} above

inline int setQbar(double A, double B, double Z1, double Z2, double R, 
		   double Dx, double Phi, int Lside, int Rside)
{
  float a=A; float b=B; float z1=Z1; float z2=Z2; float r=R;
  float dx=Dx; float phi=Phi; 
  int id=set_qbar_(&a, &b, &z1, &z2, &r, &dx, &phi, &Lside, &Rside);
  return id;
}

// add i-TOP expansion volume

inline void addExpansionVolume(int QbarID, int LR, int Shape, double Dz, double Yup,
			       double Ydown)
{
  float dz=Dz; float yup=Yup; float ydn=Ydown;
  set_extvol_(&QbarID, &LR, &Shape, &dz, &yup, &ydn);
}

// re-arrange PMT's at (both) exit window(s) 

inline void arrangePMT(int QbarID, double sizX, double sizY, double Dx=0, double Dy=0)
{
  float sizx=sizX; float sizy=sizY; float dx=Dx; float dy=Dy;
  int LR=0;
  arrange_pmt_(&QbarID, &LR, &sizx, &sizy, &dx, &dy);
  LR=1;
  arrange_pmt_(&QbarID, &LR, &sizx, &sizy, &dx, &dy);
}

// re-arrange PMT's at exit window LR 

inline void arrangePMT(int QbarID, int LR, double sizX, double sizY, 
		       double Dx=0, double Dy=0)
{
  float sizx=sizX; float sizy=sizY; float dx=Dx; float dy=Dy;
  arrange_pmt_(&QbarID, &LR, &sizx, &sizy, &dx, &dy);
}

// set mirror radius [cm]

inline void setMirrorRadius(int QbarID, double R)
{
  float r=R;
  set_rmi_(&QbarID, &r);
}

// set mirror center to Xc, Yc [cm] (Qbar frame) 

inline void setMirrorCenter(int QbarID, double Xc, double Yc)
{
  float xc=Xc; float yc=Yc;
  set_xyc_(&QbarID, &xc, &yc);
}

// define TDC
// NBIT: number of bits; ChWid: channel width [ns]; Offset: offset [ns]

inline void setTDC(int NBIT, double ChWid, double Offset=0)
{
  float chwid=ChWid; float offset=Offset;
  set_tdc_(&NBIT, &chwid, &offset);
}

// finalize TOP configuration (must be called last)

inline void TOPfinalize(int Dump=1)
{
  top_geo_finalize_(&Dump);
}

#endif

