//*****************************************************************************
//-----------------------------------------------------------------------------
// Fast simulation and reconstruction package for TOP counter (F77 core)
// M. Staric, April-2009
//-----------------------------------------------------------------------------
//
// RandomParticle.cc
// C++ interface to F77 functions: random particle
//-----------------------------------------------------------------------------
//*****************************************************************************

#include <iostream>
#include <iomanip>
#include "RandomParticle.h"
#include <math.h>
#include <stdlib.h>

extern "C" {void ranlux_(float*,int*);}
inline double ranlux()
{
  float u; int i=1;
  ranlux_(&u, &i);
  return double(u);
}

RandomParticle::RandomParticle()
{
  m_X=0;
  m_Y=0;
  m_Z=0;
  m_Tstart=0;
  m_Tstop=1.e9;
  m_Px=0;
  m_Py=0;
  m_Pz=0;
  m_LUND=0;
  m_REF=0;
  m_pmin=0;
  m_pmax=5;
  m_cthmin=-1;
  m_cthmax=1;
  m_frac[0]=0;
  m_frac[1]=0;
  m_frac[2]=.5;
  m_frac[3]=1;
  m_frac[4]=1;
  pi=4*atan(1);
  genNext();
}

RandomParticle::RandomParticle(double p, double thetaDeg, double phiDeg, int LUND)
{
  pi=4*atan(1);
  double theta=thetaDeg/180*pi;
  double phi=phiDeg/180*pi;
  m_X=0;
  m_Y=0;
  m_Z=0;
  m_Tstart=0;
  m_Tstop=1.e9;
  m_Px=p*cos(phi)*sin(theta);
  m_Py=p*sin(phi)*sin(theta);
  m_Pz=p*cos(theta);
  m_LUND=LUND;
  m_REF=1;
  m_pmin=0;
  m_pmax=5;
  m_cthmin=-1;
  m_cthmax=1;
  m_frac[0]=0;
  m_frac[1]=0;
  m_frac[2]=.5;
  m_frac[3]=1;
  m_frac[4]=1;
}

void RandomParticle::setPmin(double pmin) 
{
  m_pmin=pmin;
  genNext(); m_REF--;
}

void RandomParticle::setPmax(double pmax) 
{
  m_pmax=pmax;
  genNext(); m_REF--;
}

void RandomParticle::setCosThmin(double cthmin) 
{
  if(cthmin<-1) cthmin=-1;
  if(cthmin>1) cthmin=1;
  m_cthmin=cthmin;
  genNext(); m_REF--;
}

void RandomParticle::setCosThmax(double cthmax) 
{
  if(cthmax<-1) cthmax=-1;
  if(cthmax>1) cthmax=1;
  m_cthmax=cthmax;
  genNext(); m_REF--;
}

void RandomParticle::setFrac(double e, double mu, double pi, double K, double p)
{
  double s=e+mu+pi+K+p;
  if(s==0) return;
  m_frac[0]=e/s;
  m_frac[1]=(e+mu)/s;
  m_frac[2]=(e+mu+pi)/s;
  m_frac[3]=(e+mu+pi+K)/s;
  m_frac[4]=(e+mu+pi+K+p)/s;
  genNext(); m_REF--;
}

void RandomParticle::genNext()
{
  double p=m_pmin+(m_pmax-m_pmin)*ranlux();
  double cth=m_cthmin+(m_cthmax-m_cthmin)*ranlux();
  double sth=sqrt(1-cth*cth);
  double phi=2*pi*ranlux();
  double cfi=cos(phi);
  double sfi=sin(phi);
  m_Px=p*cfi*sth;
  m_Py=p*sfi*sth;
  m_Pz=p*cth;
  int lundc[5]={-11,-13,211,321,2212};
  double rnd=ranlux();
  for(int i=0; i<5; i++){
    if(rnd < m_frac[i]) {m_LUND=lundc[i]; break;}
  }
  if(ranlux() < 0.5) m_LUND=-m_LUND;
  m_REF++;
}

double RandomParticle::p(){return sqrt(m_Px*m_Px+m_Py*m_Py+m_Pz*m_Pz);}

double RandomParticle::theta(){return acos(m_Pz/p());}

double RandomParticle::phi(){return atan2(m_Py,m_Px);}

int RandomParticle::Q()
{
  return (abs(m_LUND)>100?1:-1)*(m_LUND>0?1:-1);
}

int RandomParticle::Hyp()
{
  int lundc[5]={11,13,211,321,2212};
  for(int i=0; i<5; i++){
    if(abs(m_LUND)==lundc[i]) return i+1;
  }	
  return 0;
}

void RandomParticle::Dump()
{
  using namespace std;
  cout << "RandomParticle::Dump(): Ref="<< m_REF <<"  Lund="<< m_LUND;
  cout <<"  p="<< setprecision(2) << p() <<"GeV/c";
  cout <<"  theta="<< setprecision(2) << theta()/pi*180;
  cout <<"  phi="<< setprecision(2) << phi()/pi*180;
  cout << endl;
}
